﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel;

namespace AddressBook
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private ContactCollection contacts;
        private BitmapImage searchBitmap;
        private BitmapImage cancelSearchBitmap;
        private bool searching;

        public MainWindow()
        {
            InitializeComponent();
            this.searchBitmap = new BitmapImage(new Uri("pack://application:,,,/AddressBook;component/Resources/search.gif"));
            this.searchBitmap.CacheOption = BitmapCacheOption.OnLoad;

            this.cancelSearchBitmap = new BitmapImage(new Uri("pack://application:,,,/AddressBook;component/Resources/cnclsearch.gif"));
            this.cancelSearchBitmap.CacheOption = BitmapCacheOption.OnLoad;

            this.contacts = new ContactCollection();
            this.contacts.Add(new Contact()
            {
                FirstName = "Scott",
                LastName = "Dorman",
                Nickname = "Scott",
                DisplayAsStyle = AddressBook.DisplayAsStyle.LastNameFirstName,

                PersonalInformation = new PersonalInformation()
                {
                    EmailAddress = "me@example.com",
                    WindowsLiveID = "me",
                    HomePhone = new PhoneNumber()
                    {
                        CountryCode = 1,
                        AreaCode = 555,
                        SubscriberNumber = "555-5555",
                    },
                    MailingAddress = new Address()
                    {
                        Street = "123 Main Street",
                        City = "Anywhere",
                        StateProvince = "AL",
                        PostalCode = "55555",
                        Country = "United States"
                    }
                },

                XfnRelationships = new Xfn.XfnRelationships()
                {
                    Family = Xfn.Family.None,
                    Geographic = Xfn.Geographic.Neighbor,
                    Me = true,
                }
            });
            this.contacts.Add(new Contact()
            {
                FirstName = "Linus",
                LastName = "Torvalds",
            });
            this.contacts.Add(new Contact()
            {
                FirstName = "Bill",
                LastName = "Gates",
            });
            this.contacts.Add(new Contact()
            {
                FirstName = "Steve",
                LastName = "Ballmer",
            });
            this.contacts.Add(new Contact()
            {
                FirstName = "Mikhail",
                LastName = "Baryshnikov",
            });
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            ContactEditor editorWindow = new ContactEditor();
            bool? result = editorWindow.ShowDialog();
        }

        private void AdvancedSearch_Click(object sender, RoutedEventArgs e)
        {
            if (this.advancedSearchPanel.Visibility == System.Windows.Visibility.Collapsed)
            {
                this.advancedSearchPanel.Visibility = System.Windows.Visibility.Visible;
            }
            else
            {
                this.advancedSearchPanel.Visibility = System.Windows.Visibility.Collapsed;
            }
        }

        private void Search_Click(object sender, RoutedEventArgs e)
        {
            if (this.searching)
            {
                this.searching = false;
                this.searchButtonImage.BeginInit();
                this.searchButtonImage.Source = cancelSearchBitmap;
                this.searchButtonImage.EndInit();

                if (this.searchCategory.IsChecked.HasValue && this.searchCategory.IsChecked.Value == true)
                {
                    this.DataContext = contacts.
                        Where(c => c.Categories.Contains(this.categories.SelectedValue.ToString()) && (c.FirstName.Contains(this.searchCriteria.Text) || c.LastName.Contains(this.searchCriteria.Text))).
                        ToList();
                }
                else
                {
                    this.DataContext = contacts.Where(c => c.FirstName.Contains(this.searchCriteria.Text) || c.LastName.Contains(this.searchCriteria.Text)).ToList();
                }
            }
            else
            {
                this.DataContext = this.Contacts;
                this.searchCriteria.Text = String.Empty;
                this.searchButtonImage.BeginInit();
                this.searchButtonImage.Source = searchBitmap;
                this.searchButtonImage.EndInit();
            }
        }

        public ContactCollection Contacts
        {
            get
            {
                return this.contacts;
            }
        }

        private void searchCriteria_TextChanged(object sender, TextChangedEventArgs e)
        {
            TextBox textBox = e.OriginalSource as TextBox;
            if (textBox != null)
            {
                if (String.IsNullOrWhiteSpace(textBox.Text))
                {
                    this.searchButton.IsEnabled = false;
                    this.searching = false;
                }
                else
                {
                    this.searchButton.IsEnabled = true;
                    this.searching = true;
                }
            }
        }
        
        private void Button_Click_1(object sender, RoutedEventArgs e)
        {
            CollectionViewSource view = this.DataContext as CollectionViewSource;
            // the order of the items is the same as in the enumeration declaration
            view.SortDescriptions.Add(
                new SortDescription("XfnRelationships.Me", ListSortDirection.Descending));
            //view.SortDescriptions.Add(
            //    new SortDescription("StartDate", ListSortDirection.Ascending));
        }
    }
}
