using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using AddressBook.Xfn;
using System.Xml.Linq;

namespace AddressBook
{
    public class Contact
    {
        private List<string> categories;

        public Contact()
        {
        }

        public string FirstName { get; set; }
        public string LastName { get; set; }
        public string Nickname { get; set; }
        public DisplayAsStyle DisplayAsStyle { get; set; }

        public string DisplayAs
        {
            get
            {
                if (this.DisplayAsStyle == AddressBook.DisplayAsStyle.FirstNameLastName)
                {
                    return String.Format("{0} {1}", this.FirstName, this.LastName);
                }
                else
                {
                    return String.Format("{0}, {1}", this.LastName, this.FirstName);
                }
            }
        }

        public PersonalInformation PersonalInformation { get; set; }
        public BusinessInformation BusinessInformation { get; set; }

        public string OtherEmailAddress { get; set; }
        public PreferredEmail PreferredEmail { get; set; }
        public PhoneNumber OtherPhoneNumber { get; set; }
        public Uri Website { get; set; }
        public DateTime BirthDate { get; set; }
        public string Notes { get; set; }
        
        public List<string> Categories
        {
            get
            {
                return this.categories;
            }
        }

        public XfnRelationships XfnRelationships { get; set; }

        public void Save()
        {
            XElement document = new XElement("contact",
               new XElement("firstName", this.FirstName),
               new XElement("lastName", this.LastName),
               new XElement("nickName", this.Nickname),
               new XElement("displayAsStyle", this.DisplayAsStyle),
               new XElement("otherEmailAddress", this.OtherEmailAddress),
               new XElement("preferredEmail", this.PreferredEmail),
               new XElement("website", this.Website),
               new XElement("birthDate", this.BirthDate),
               new XElement("personalInformation",
                   new XElement("emailAddress", this.PersonalInformation.EmailAddress),
                   new XElement("homePhone",
                       new XElement("countryCode", this.PersonalInformation.HomePhone.CountryCode),
                       new XElement("areaCode", this.PersonalInformation.HomePhone.AreaCode),
                       new XElement("subscriberNumber", this.PersonalInformation.HomePhone.SubscriberNumber),
                       new XElement("extension", this.PersonalInformation.HomePhone.Extension)),
                   new XElement("mailingAddress",
                       new XElement("street", this.PersonalInformation.MailingAddress.Street),
                       new XElement("city", this.PersonalInformation.MailingAddress.City),
                       new XElement("stateProvince", this.PersonalInformation.MailingAddress.StateProvince),
                       new XElement("postalCode", this.PersonalInformation.MailingAddress.PostalCode),
                       new XElement("country", this.PersonalInformation.MailingAddress.Country)),
                   new XElement("mobilePhone",
                       new XElement("countryCode", this.PersonalInformation.MobilePhone.CountryCode),
                       new XElement("areaCode", this.PersonalInformation.MobilePhone.AreaCode),
                       new XElement("subscriberNumber", this.PersonalInformation.MobilePhone.SubscriberNumber),
                       new XElement("extension", this.PersonalInformation.MobilePhone.Extension)),
                   new XElement("useLiveIdWithMessenger", this.PersonalInformation.UseLiveIdWithMessenger),
                   new XElement("windowsLiveID", this.PersonalInformation.WindowsLiveID)),
                 new XElement("businessInformation",
                   new XElement("company", this.BusinessInformation.Company),
                   new XElement("emailAddress", this.BusinessInformation.EmailAddress),
                   new XElement("phone",
                       new XElement("countryCode", this.BusinessInformation.Phone.CountryCode),
                       new XElement("areaCode", this.BusinessInformation.Phone.AreaCode),
                       new XElement("subscriberNumber", this.BusinessInformation.Phone.SubscriberNumber),
                       new XElement("extension", this.BusinessInformation.Phone.Extension)),
                   new XElement("mailingAddress",
                       new XElement("street", this.BusinessInformation.MailingAddress.Street),
                       new XElement("city", this.BusinessInformation.MailingAddress.City),
                       new XElement("stateProvince", this.BusinessInformation.MailingAddress.StateProvince),
                       new XElement("postalCode", this.BusinessInformation.MailingAddress.PostalCode),
                       new XElement("country", this.BusinessInformation.MailingAddress.Country)),
                   new XElement("pager",
                       new XElement("countryCode", this.BusinessInformation.Pager.CountryCode),
                       new XElement("areaCode", this.BusinessInformation.Pager.AreaCode),
                       new XElement("subscriberNumber", this.BusinessInformation.Pager.SubscriberNumber),
                       new XElement("extension", this.BusinessInformation.Pager.Extension)),
                 new XElement("fax",
                       new XElement("countryCode", this.BusinessInformation.Fax.CountryCode),
                       new XElement("areaCode", this.BusinessInformation.Fax.AreaCode),
                       new XElement("subscriberNumber", this.BusinessInformation.Fax.SubscriberNumber),
                       new XElement("extension", this.BusinessInformation.Fax.Extension))),
                new XElement("xfn",
                    new XElement("family", this.XfnRelationships.Family),
                    new XElement("friendship", this.XfnRelationships.Friendship),
                    new XElement("geographic", this.XfnRelationships.Geographic),
                    new XElement("me", this.XfnRelationships.Me),
                    new XElement("met", this.XfnRelationships.Met),
                    new XElement("professional", this.XfnRelationships.Professional),
                    new XElement("romantic", this.XfnRelationships.Romantic)));

            document.Save(this.LastName + ", " + this.FirstName + ".contact");
        }
    }
}
