﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;
using System.IO;
using System.ComponentModel;

namespace PhotoViewer
{
    public class PhotoCollection : ObservableCollection<IPhoto>
    {
        private DirectoryInfo directoryInfo;

        public PhotoCollection(string directory)
            : base()
        {
            if (directory == null)
            {
                throw new ArgumentNullException("directory");
            }

            if (String.IsNullOrWhiteSpace(directory))
            {
                throw new ArgumentException();
            }

            this.Path = directory;
        }

        public string Path
        {
            get
            {
                return this.directoryInfo.FullName;
            }
            set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("value");
                }

                if (String.IsNullOrWhiteSpace(value))
                {
                    throw new ArgumentException();
                }

                this.directoryInfo = new DirectoryInfo(value);
                this.OnPropertyChanged(new PropertyChangedEventArgs("Path"));
                this.Update();
            }
        }

        private void Update()
        {
            this.Clear();
            if (this.directoryInfo.Exists)
            {
                try
                {
                    foreach (FileInfo f in directoryInfo.EnumerateFiles("*.jpg"))
                    {
                        Add(new Photo(f.FullName));
                    }
                }
                catch (DirectoryNotFoundException)
                {
                    System.Windows.MessageBox.Show("No Such Directory");
                }
            }
        }
    }
}
