﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Media.Imaging;
using PhotoViewer.Exif;

namespace PhotoViewer
{
    public class Photo : IPhoto
    {
        private bool exists;
        private BitmapFrame image;
        private ExifMetadata metadata;
        private Uri source;


        public Photo(string path)
            : this(new Uri(path))
        {
        }

        public Photo(Uri path)
        {
            if (path == null)
            {
                throw new ArgumentNullException("path");
            }

            try
            {
                if (path.IsFile)
                {
                    this.source = path;
                    this.Refresh();
                }
            }
            catch (InvalidOperationException)
            {
                this.source = null;
                this.Refresh();
            }
        }

        public bool Exists
        {
            get
            {
                return this.exists;
            }
        }

        public BitmapFrame Image
        {
            get
            {
                return this.image;
            }
        }

        public ExifMetadata Metadata
        {
            get
            {
                return this.metadata;
            }
        }

        public Uri Source
        {
            get
            {
                return this.source;
            }
        }

        public void Refresh()
        {
            if (this.source != null)
            {
                this.image = BitmapFrame.Create(this.source);
                this.metadata = new ExifMetadata(this.source);
                this.exists = true;
            }
            else
            {
                this.image = null;
                this.metadata = new ExifMetadata();
                this.exists = false;
            }
        }

        public override string ToString()
        {
            if (this.source != null)
            {
                return this.source.ToString();
            }

            return String.Empty;
        }
    }
}
