﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Globalization;

namespace PhotoViewer.Exif
{
    public struct ExifMetadata
    {
        private string cameraModel;
        private ushort? colorRepresentation;
        private string creationSoftware;
        private string dateTaken;
        private string equipmentManufacturer;
        private decimal? exposureCompensation;
        private ushort? exposureMode;
        private decimal? exposureTime;
        private ushort? flashMode;
        private decimal? focalLength;
        private uint? height;
        private decimal? horizontalResolution;
        private ushort? isoSpeed;
        private decimal? lensAperature;
        private ushort? lightSource;
        private decimal? verticalResolution;
        private uint? width;

        public ExifMetadata(Uri imageUri)
        {
            this.cameraModel = null;
            this.colorRepresentation = null;
            this.creationSoftware = null;
            this.dateTaken = null;
            this.equipmentManufacturer = null;
            this.exposureCompensation = null;
            this.exposureMode = null;
            this.exposureTime = null;
            this.flashMode = null;
            this.focalLength = null;
            this.height = null;
            this.horizontalResolution = null;
            this.isoSpeed = null;
            this.lensAperature = null;
            this.lightSource = null;
            this.verticalResolution = null;
            this.width = null;
        }

        public string CameraModel
        {
            get
            {
                return this.cameraModel;
            }
        }

        public ColorRepresentation ColorRepresentation
        {
            get
            {
                ColorRepresentation value = Exif.ColorRepresentation.Uncalibrated;
                if (this.colorRepresentation.HasValue)
                {
                    if (!String.IsNullOrWhiteSpace(Enum.GetName(typeof(ColorRepresentation), this.colorRepresentation)))
                    {
                        value = (ColorRepresentation)this.colorRepresentation;
                    }
                }

                return value;
            }
        }

        public string CreationSoftware
        {
            get
            {
                return this.creationSoftware;
            }
        }

        public DateTime? DateTaken
        {
            get
            {
                DateTime? result = null;
                if (!String.IsNullOrWhiteSpace(this.dateTaken))
                {
                    DateTime date;
                    //"yyyy:MM:dd HH:mm:ss", 
                    if (DateTime.TryParse(this.dateTaken, DateTimeFormatInfo.InvariantInfo, DateTimeStyles.None, out date))
                    {
                        result = date;
                    }
                }

                return result;
            }
        }

        public string EquipmentManufacturer
        {
            get
            {
                return this.equipmentManufacturer;
            }
        }

        public decimal? ExposureCompensation
        {
            get
            {
                return this.exposureCompensation;
            }
        }

        public ExposureMode ExposureMode
        {
            get
            {
                ExposureMode value = Exif.ExposureMode.Unknown;
                if (this.exposureMode.HasValue)
                {
                    if (!String.IsNullOrWhiteSpace(Enum.GetName(typeof(ExposureMode), this.exposureMode)))
                    {
                        value = (ExposureMode)this.exposureMode;
                    }
                }

                return value;
            }
        }

        public decimal? ExposureTime
        {
            get
            {
                return this.exposureTime;
            }
        }

        public FlashMode FlashMode
        {
            get
            {
                FlashMode value = FlashMode.FlashDidNotFire;
                if (this.flashMode.HasValue && this.flashMode % 2 == 1)
                {
                    value = FlashMode.FlashFired;
                }

                return value;
            }
        }

        public decimal? FocalLength
        {
            get
            {
                return this.focalLength;
            }
        }

        public uint? Height
        {
            get
            {
                return this.height;
            }
        }

        public decimal? HorizontalResolution
        {
            get
            {
                return this.horizontalResolution;
            }
        }

        public ushort? IsoSpeed
        {
            get
            {
                return this.isoSpeed;
            }
        }

        public decimal? LensAperture
        {
            get
            {
                return this.lensAperature;
            }
        }

        public LightSource LightSource
        {
            get
            {
                LightSource value = Exif.LightSource.Unknown;
                if (this.lightSource.HasValue)
                {
                    if (!String.IsNullOrWhiteSpace(Enum.GetName(typeof(LightSource), this.lightSource)))
                    {
                        value = (LightSource)this.lightSource;
                    }
                }

                return value;
            }
        }

        public decimal? VerticalResolution
        {
            get
            {
                return this.verticalResolution;
            }
        }

        public uint? Width
        {
            get
            {
                return this.width;
            }
        }
    }
}
